@@ -25,15 +25,16 @@
  */
 
 
-#ifdef GLX_DIRECT_RENDERING
-
-#include <inttypes.h>
 #include <assert.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include <sys/mman.h>
 #include <stdio.h>
 
+#ifndef MAP_FAILED
+#define MAP_FAILED ((void *)-1)
+#endif
+
 #ifndef DRI_NEW_INTERFACE_ONLY
 # include <X11/Xlibint.h>
 # include <Xext.h>
@@ -50,7 +51,7 @@
 #include "drm_sarea.h"
 #include "glcontextmodes.h"
 
-#ifndef PFNGLXGETMSCRATEOMLPROC
+#ifndef GLX_OML_sync_control
 typedef GLboolean ( * PFNGLXGETMSCRATEOMLPROC) (__DRInativeDisplay *dpy, __DRIid drawable, int32_t *numerator, int32_t *denominator);
 #endif
 
@@ -232,34 +233,13 @@ static GLboolean __driAddDrawable(void *drawHash, __DRIdrawable *pdraw)
 static __DRIdrawable *__driFindDrawable(void *drawHash, __DRIid draw)
 {
     int retcode;
-    union 
-    {
-	__DRIdrawable *pdraw;
-	void *ptr;
-    } p;
+    __DRIdrawable *pdraw;
 
-    retcode = drmHashLookup(drawHash, draw, &p.ptr);
+    retcode = drmHashLookup(drawHash, draw, (void *)&pdraw);
     if (retcode)
 	return NULL;
 
-    return p.pdraw;
-}
-
-static void __driRemoveDrawable(void *drawHash, __DRIdrawable *pdraw)
-{
-    int retcode;
-    union 
-    {
-	__DRIdrawablePrivate *pdp;
-	void *ptr;
-    } p;
-
-    p.pdp = (__DRIdrawablePrivate *)pdraw->private;
-
-    retcode = drmHashLookup(drawHash, p.pdp->draw, &p.ptr);
-    if (!retcode) { /* Found */
-	drmHashDelete(drawHash, p.pdp->draw);
-    }
+    return pdraw;
 }
 
 #ifndef DRI_NEW_INTERFACE_ONLY
@@ -317,24 +297,20 @@ static void __driGarbageCollectDrawables(void *drawHash)
 {
     __DRIid draw;
     __DRInativeDisplay *dpy;
-    union 
-    {
-	__DRIdrawable *pdraw;
-	void *ptr;
-    } p;
+    __DRIdrawable *pdraw;
 
-    if (drmHashFirst(drawHash, &draw, &p.ptr)) {
+    if (drmHashFirst(drawHash, &draw, (void *)&pdraw) == 1) {
 	do {
-	    __DRIdrawablePrivate *pdp = (__DRIdrawablePrivate *)p.pdraw->private;
+	    __DRIdrawablePrivate *pdp = (__DRIdrawablePrivate *)pdraw->private;
 	    dpy = pdp->driScreenPriv->display;
 	    if (! (*window_exists)(dpy, draw)) {
 		/* Destroy the local drawable data in the hash table, if the
 		   drawable no longer exists in the Xserver */
-		__driRemoveDrawable(drawHash, p.pdraw);
-		(*p.pdraw->destroyDrawable)(dpy, p.pdraw->private);
-		_mesa_free(p.pdraw);
+	        drmHashDelete(drawHash, draw);
+		(*pdraw->destroyDrawable)(dpy, pdraw->private);
+		_mesa_free(pdraw);
 	    }
-	} while (drmHashNext(drawHash, &draw, &p.ptr));
+	} while (drmHashNext(drawHash, &draw, (void *)&pdraw) == 1);
     }
 }
 
@@ -1314,6 +1290,7 @@ __driUtilCreateNewScreen(__DRInativeDisplay *dpy, int scrn, __DRIscreen *psc,
     psp->fbHeight = frame_buffer->height;
     psp->devPrivSize = frame_buffer->dev_priv_size;
     psp->pDevPriv = frame_buffer->dev_priv;
+    psp->fbBPP = psp->fbStride * 8 / frame_buffer->width;
 
     psp->fd = fd;
 
@@ -1597,7 +1574,7 @@ __driUtilCreateScreen(Display *dpy, int scrn, __DRIscreen *psc,
  *
  * \sa __glXGetInternalVersion().
  */
-int driCompareGLXAPIVersion( GLuint required_version )
+int driCompareGLXAPIVersion( GLint required_version )
 {
    if ( api_ver > required_version ) {
       return 1;
@@ -1711,5 +1688,3 @@ driCalculateSwapUsage( __DRIdrawablePrivate *dPriv, int64_t last_swap_ust,
 }
 
 /*@}*/
-
-#endif /* GLX_DIRECT_RENDERING */
